;; pg-display-table.el --- Wrapper functions for display-table vectors and
;; objects.
;;
;; Copyright (C) 2009 LFCS Edinburgh.
;; Author:     Jerry James
;; License:    GPL (GNU GENERAL PUBLIC LICENSE)
;;
;; pg-display-table.el,v 1.0 2009/07/06 16:07:09 da Exp
;;
;;
;;; Commentary:
;;
;; This file hides the differences between display-tables that are represented
;; as vectors and recent XEmacs display-tables, which are objects.

;;; Code:

(require 'disp-table)

(defmacro defun-when-void (&rest args)
  "Define a function, just like `defun', unless it's already defined.
Used for compatibility among different emacs variants."
  `(if (fboundp ',(car args))
       nil
     (defun ,@args)))

(defun-when-void put-display-table (range value display-table)
  "Set the value for char RANGE to VALUE in DISPLAY-TABLE."
  (if (sequencep display-table)
      (aset display-table range value)
    (put-char-table range value display-table)))

(defun-when-void get-display-table (character display-table)
  "Find value for CHARACTER in DISPLAY-TABLE."
  (if (sequencep display-table)
      (aref display-table character)
    (get-char-table character display-table)))

(defun-when-void fill-display-table (value display-table)
  "Map every entry in the table to VALUE."
  (if (sequencep display-table)
      (fillarray display-table value)
    (put-char-table t value display-table)))

(provide 'pg-display-table)
;;; pg-display-table.el ends here
